@@ -68,10 +68,11 @@ static void rb_station_properties_dialog_update (RBStationPropertiesDialog *dial
 static void rb_station_properties_dialog_update_title_entry (RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_update_genre (RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_update_play_count (RBStationPropertiesDialog *dialog);
+static void rb_station_properties_dialog_update_bitrate (RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_update_last_played (RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_update_rating (RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_rated_cb (RBRating *rating,
-						   int score,
+						   double score,
 						   RBStationPropertiesDialog *dialog);
 static void rb_station_properties_dialog_sync_entries (RBStationPropertiesDialog *dialog);
 
@@ -86,6 +87,7 @@ struct RBStationPropertiesDialogPrivate
 	GtkWidget   *location;
 	GtkWidget   *lastplayed;
 	GtkWidget   *playcount;
+	GtkWidget   *bitrate;
 	GtkWidget   *rating;
 	GtkWidget   *okbutton;
 	GtkWidget   *cancelbutton;
@@ -156,10 +158,10 @@ rb_station_properties_dialog_init (RBStationPropertiesDialog *dialog)
 	
 	dialog->priv = g_new0 (RBStationPropertiesDialogPrivate, 1);
 	
-	g_signal_connect (G_OBJECT (dialog),
-			  "response",
-			  G_CALLBACK (rb_station_properties_dialog_response_cb),
-			  dialog);
+	g_signal_connect_object (G_OBJECT (dialog),
+				 "response",
+				 G_CALLBACK (rb_station_properties_dialog_response_cb),
+				 dialog, 0);
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
@@ -191,6 +193,7 @@ rb_station_properties_dialog_init (RBStationPropertiesDialog *dialog)
 
 	dialog->priv->lastplayed = glade_xml_get_widget (xml, "lastplayedLabel");
 	dialog->priv->playcount = glade_xml_get_widget (xml, "playcountLabel");
+	dialog->priv->bitrate = glade_xml_get_widget (xml, "bitrateLabel");
 
 	dialog->priv->rating = GTK_WIDGET (rb_rating_new ());
 	g_signal_connect_object (dialog->priv->rating, 
@@ -317,6 +320,7 @@ rb_station_properties_dialog_update (RBStationPropertiesDialog *dialog)
 	rb_station_properties_dialog_update_title_entry (dialog);
 	rb_station_properties_dialog_update_genre (dialog);
 	rb_station_properties_dialog_update_play_count (dialog);
+	rb_station_properties_dialog_update_bitrate (dialog);
 	rb_station_properties_dialog_update_last_played (dialog);
 	rb_station_properties_dialog_update_rating (dialog);
 }
@@ -380,7 +384,7 @@ rb_station_properties_dialog_update_location (RBStationPropertiesDialog *dialog)
 
 static void
 rb_station_properties_dialog_rated_cb (RBRating *rating,
-				       int score,
+				       double score,
 				       RBStationPropertiesDialog *dialog)
 {
 	GValue value = { 0, };
@@ -390,8 +394,8 @@ rb_station_properties_dialog_rated_cb (RBRating *rating,
 	g_return_if_fail (score >= 0 && score <= 5 );
 
 	/* set the new value for the song */
-	g_value_init (&value, G_TYPE_INT);
-	g_value_set_int (&value, score);
+	g_value_init (&value, G_TYPE_DOUBLE);
+	g_value_set_double (&value, score);
 	rhythmdb_write_lock (dialog->priv->db);
 	rhythmdb_entry_set (dialog->priv->db,
 			    dialog->priv->current_entry,
@@ -421,6 +425,26 @@ rb_station_properties_dialog_update_play_count (RBStationPropertiesDialog *dialo
 	g_free (text);
 }
 
+static void
+rb_station_properties_dialog_update_bitrate (RBStationPropertiesDialog *dialog)
+{
+	int val;
+	char *text;
+
+	rhythmdb_read_lock (dialog->priv->db);
+	val = rhythmdb_entry_get_int (dialog->priv->db,
+				      dialog->priv->current_entry,
+				      RHYTHMDB_PROP_BITRATE);
+	rhythmdb_read_unlock (dialog->priv->db);
+	if (val == 0)
+		text = g_strdup (_("Unknown"));
+	else
+		text = g_strdup_printf ("%d", val);
+
+	gtk_label_set_text (GTK_LABEL (dialog->priv->bitrate), text);
+	g_free (text);
+}
+
 static void
 rb_station_properties_dialog_update_last_played (RBStationPropertiesDialog *dialog)
 {
@@ -438,14 +462,14 @@ rb_station_properties_dialog_update_last_played (RBStationPropertiesDialog *dial
 static void
 rb_station_properties_dialog_update_rating (RBStationPropertiesDialog *dialog)
 {
-	guint rating;
+	double rating;
 
 	g_return_if_fail (RB_IS_STATION_PROPERTIES_DIALOG (dialog));
 
 	rhythmdb_read_lock (dialog->priv->db);
-	rating = rhythmdb_entry_get_int (dialog->priv->db,
-					 dialog->priv->current_entry,
-					 RHYTHMDB_PROP_RATING);
+	rating = rhythmdb_entry_get_double (dialog->priv->db,
+					    dialog->priv->current_entry,
+					    RHYTHMDB_PROP_RATING);
 	rhythmdb_read_unlock (dialog->priv->db);
 
 	g_object_set (G_OBJECT (dialog->priv->rating),
