@@ -56,6 +56,7 @@ static struct
 	{"application/x-id3", "id3tag"},
 	{"application/ogg", NULL},
 	{"audio/x-flac", "flactag"},
+	{"application/x-ape", NULL},
 };
 
 static GObjectClass *parent_class = NULL;
@@ -397,8 +398,8 @@ rb_metadata_load (RBMetaData *md,
 	 */
 	pipeline = gst_pipeline_new ("pipeline");
 
-	g_signal_connect (pipeline, "error", G_CALLBACK (rb_metadata_gst_error_cb), md);
-	g_signal_connect (pipeline, "found-tag", G_CALLBACK (rb_metadata_gst_found_tag), md);
+	g_signal_connect_object (pipeline, "error", G_CALLBACK (rb_metadata_gst_error_cb), md, 0);
+	g_signal_connect_object (pipeline, "found-tag", G_CALLBACK (rb_metadata_gst_found_tag), md, 0);
 
 	plugin_name = "gnomevfssrc";
 	if (!(gnomevfssrc = gst_element_factory_make (plugin_name, plugin_name)))
@@ -408,7 +409,7 @@ rb_metadata_load (RBMetaData *md,
 	plugin_name = "typefind";
 	if (!(typefind = gst_element_factory_make (plugin_name, plugin_name)))
 		goto missing_plugin;
-	g_signal_connect (typefind, "have_type", G_CALLBACK (rb_metadata_gst_typefind_cb), md);
+	g_signal_connect_object (typefind, "have_type", G_CALLBACK (rb_metadata_gst_typefind_cb), md, 0);
 	gst_bin_add (GST_BIN (pipeline), typefind);		  
 	plugin_name = "spider";
 	if (!(spider = gst_element_factory_make (plugin_name, plugin_name)))
@@ -419,8 +420,8 @@ rb_metadata_load (RBMetaData *md,
 		goto missing_plugin;
 	gst_bin_add (GST_BIN (pipeline), md->priv->sink);		  
 	g_object_set (G_OBJECT (md->priv->sink), "signal-handoffs", TRUE, NULL);
-	g_signal_connect (md->priv->sink, "handoff", G_CALLBACK (rb_metadata_gst_fakesink_handoff_cb), md);
-	g_signal_connect (md->priv->sink, "eos", G_CALLBACK (rb_metadata_gst_eos_cb), md);
+	g_signal_connect_object (md->priv->sink, "handoff", G_CALLBACK (rb_metadata_gst_fakesink_handoff_cb), md, 0);
+	g_signal_connect_object (md->priv->sink, "eos", G_CALLBACK (rb_metadata_gst_eos_cb), md, 0);
 
 	gst_element_link_many (gnomevfssrc, typefind, spider, NULL);
 
@@ -469,14 +470,24 @@ rb_metadata_load (RBMetaData *md,
 		goto out;
 	}
 
-	/* FIXME
-	 * For now, we simply ignore files with an unknown MIME
-	 * type. This will be fixed once GStreamer gives us
-	 * a good way to detect audio. */
-	if (!rb_metadata_can_save (md, md->priv->type)) {
-		rb_debug ("ignoring file %s with detected type %s",
-			  uri, md->priv->type);
-		goto out;
+	{
+		/* FIXME
+		 * For now, we simply ignore files with an unknown MIME
+		 * type. This will be fixed once GStreamer gives us
+		 * a good way to detect audio. */
+		gboolean supported_type = FALSE;
+		guint i;
+
+		for (i = 0; i < G_N_ELEMENTS (rb_metadata_type_map); i++)
+			if (!strcmp (rb_metadata_type_map[i].mimetype, md->priv->type))
+				supported_type = TRUE;
+		if (!supported_type) {
+			rb_debug ("ignoring file %s with detected type %s",
+				  uri, md->priv->type);
+			g_free (md->priv->type);
+			md->priv->type = NULL;
+			goto out;
+		}
 	}
 	
 	if (md->priv->error) {
@@ -548,7 +559,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 
 	pipeline = gst_pipeline_new ("pipeline");
 
-	g_signal_connect (pipeline, "error", G_CALLBACK (rb_metadata_gst_error_cb), md);
+	g_signal_connect_object (pipeline, "error", G_CALLBACK (rb_metadata_gst_error_cb), md, 0);
 
 	plugin_name = "gnomevfssrc";
 	if (!(gnomevfssrc = gst_element_factory_make (plugin_name, plugin_name)))
@@ -583,7 +594,7 @@ rb_metadata_save (RBMetaData *md, GError **error)
 		goto missing_plugin;
 	gst_bin_add (GST_BIN (pipeline), md->priv->sink);		  
 	g_object_set (G_OBJECT (md->priv->sink), "handle", handle, NULL);
-	g_signal_connect (md->priv->sink, "eos", G_CALLBACK (rb_metadata_gst_eos_cb), md);
+	g_signal_connect_object (md->priv->sink, "eos", G_CALLBACK (rb_metadata_gst_eos_cb), md, 0);
 
 	gst_element_link_many (gnomevfssrc, tagger, md->priv->sink, NULL);
 
