@@ -57,6 +57,8 @@ struct RBPlayerPrivate
 
 	GstElement *decoder;
 	GstElement *volume;
+	GstElement *audioconvert;
+	GstElement *audioscale;
 	GstElement *sink;
 
 	GError *error;
@@ -72,7 +74,6 @@ struct RBPlayerPrivate
 
 	GstDParam *volume_dparam;
 	float cur_volume;
-	gboolean mute;
 
 	GTimer *timer;
 	long timer_add;
@@ -86,6 +87,7 @@ typedef enum
 	INFO,
 	BUFFERING_BEGIN,
 	BUFFERING_END,
+	BUFFERING_PROGRESS,
 	ERROR,
 	TICK,
 	LAST_SIGNAL
@@ -177,6 +179,16 @@ rb_player_class_init (RBPlayerClass *klass)
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE,
 			      0);
+	rb_player_signals[BUFFERING_PROGRESS] =
+		g_signal_new ("buffering_progress",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPlayerClass, buffering_progress),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__INT,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_INT);
 	rb_player_signals[ERROR] =
 		g_signal_new ("error",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -313,21 +325,7 @@ eos_cb (GstElement *element,
 	RBPlayer *mp)
 {
 	g_object_ref (G_OBJECT (mp));
-
-	if (gst_element_set_state (mp->priv->sink, GST_STATE_NULL) != GST_STATE_SUCCESS) {
-		RBPlayerSignal *signal;
-		
-		signal = g_new0 (RBPlayerSignal, 1);
-		signal->object = mp;
-		signal->error = g_error_new_literal (RB_PLAYER_ERROR,
-						     RB_PLAYER_ERROR_GENERAL,
-						     _("Failed to close audio output sink"));
-		
-		g_object_ref (G_OBJECT (mp));
-		
-		g_idle_add ((GSourceFunc) error_signal_idle, signal);
-	} else
-		g_idle_add ((GSourceFunc) eos_signal_idle, mp);
+	g_idle_add ((GSourceFunc) eos_signal_idle, mp);
 }
 
 static void
@@ -423,7 +421,15 @@ rb_player_construct (RBPlayer *mp,
 		     GError **error)
 {
 	GstDParamManager *dpman;
-	char *decoder_name = NULL;
+	char *element_name = NULL;
+
+#define MAKE_ELEMENT_OR_LOSE(NAME, NICE) G_STMT_START {	\
+	element_name = #NAME ;\
+	rb_debug ("constructing element \"" #NICE "\""); \
+	mp->priv->NICE = gst_element_factory_make (#NAME, #NICE); \
+	if (!mp->priv->NICE) \
+		goto missing_element; \
+	} G_STMT_END
 
 	/* The main playback pipeline for iradio, at the end this looks like:
 	 * { { src ! queue } ! { mad ! volume ! sink } }
@@ -431,25 +437,17 @@ rb_player_construct (RBPlayer *mp,
 	 * For local files, it just looks like:
 	 *  { src ! spider ! volume ! sink }
 	 */
-	mp->priv->pipeline = gst_element_factory_make ("thread", "pipeline");
-	if (!mp->priv->pipeline) {
-		g_set_error (error,
-			     RB_PLAYER_ERROR,
-			     RB_PLAYER_ERROR_GENERAL,
-			     "%s",
-			     _("Failed to create thread element; check your GStreamer installation"));
-		return;
-	}
-	g_signal_connect (G_OBJECT (mp->priv->pipeline),
-			  "deep_notify",
-			  G_CALLBACK (deep_notify_cb),
-			  mp);
+	MAKE_ELEMENT_OR_LOSE(thread, pipeline);
+	g_signal_connect_object (G_OBJECT (mp->priv->pipeline),
+				 "deep_notify",
+				 G_CALLBACK (deep_notify_cb),
+				 mp, 0);
 
 	mp->priv->error_signal_id =
-		g_signal_connect (G_OBJECT (mp->priv->pipeline),
-				  "error",
-				  G_CALLBACK (error_cb),
-				  mp);
+		g_signal_connect_object (G_OBJECT (mp->priv->pipeline),
+					 "error",
+					 G_CALLBACK (error_cb),
+					 mp, 0);
 
 	/* Construct the two threads */
 	if (iradio_mode) {
@@ -465,18 +463,14 @@ rb_player_construct (RBPlayer *mp,
 	/* Construct elements */
 
 	/* The source */
-	mp->priv->src = audiocd_mode ? gst_element_factory_make ("cdparanoia", "src")
-		: gst_element_factory_make ("gnomevfssrc", "src");
-	if (mp->priv->src == NULL) {
-		g_set_error (error,
-			     RB_PLAYER_ERROR,
-			     RB_PLAYER_ERROR_NO_INPUT_PLUGIN,
-			     _("Failed to create %s input element; check your installation"),
-			     audiocd_mode ? "cdparanoia" : "gnomevfssrc");
-		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
-		mp->priv->pipeline = NULL;
-		return;
-	}
+
+	if (audiocd_mode)
+		element_name = "cdparanoia";
+	else
+		element_name = "gnomevfssrc";
+	mp->priv->src = gst_element_factory_make (element_name, "src");	
+	if (mp->priv->src == NULL)
+		goto missing_element;
 	gst_bin_add (GST_BIN (mp->priv->srcthread), mp->priv->src);
 
 #ifdef HAVE_AUDIOCD
@@ -495,72 +489,36 @@ rb_player_construct (RBPlayer *mp,
 	rb_debug ("constructing queue");
 	/* The queue */
 	if (iradio_mode) {
-		mp->priv->queue = gst_element_factory_make ("queue", "queue");
-		if (mp->priv->queue == NULL) {
-			g_set_error (error,
-				     RB_PLAYER_ERROR,
-				     RB_PLAYER_ERROR_NO_QUEUE_PLUGIN,
-				     _("Failed to create queue element; check your installation"));
-			gst_object_unref (GST_OBJECT (mp->priv->pipeline));
-			mp->priv->pipeline = NULL;
-			return;
-		}
+		element_name = "queue";
+		mp->priv->queue = gst_element_factory_make (element_name, element_name);
+		if (mp->priv->queue == NULL)
+			goto missing_element;
 		g_object_set (G_OBJECT (mp->priv->queue), "max-size-bytes", 64 * 1024, NULL);
-		g_signal_connect (G_OBJECT (mp->priv->queue), "overrun",
-				  G_CALLBACK (queue_full_cb), mp);
+		g_signal_connect_object (G_OBJECT (mp->priv->queue), "overrun",
+					 G_CALLBACK (queue_full_cb), mp, 0);
 		gst_bin_add (GST_BIN (mp->priv->srcthread), mp->priv->queue);
 	}
 
-	rb_debug ("constructing typefind");
-	mp->priv->typefind = gst_element_factory_make ("typefind", "typefind");
-	if (mp->priv->typefind == NULL) {
-		g_set_error (error,
-			     RB_PLAYER_ERROR,
-			     RB_PLAYER_ERROR_NO_TYPEFIND_PLUGIN,
-			     _("Failed to create typefind element; check your installation"));
-		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
-		mp->priv->pipeline = NULL;
-		return;
-	}
+	MAKE_ELEMENT_OR_LOSE(typefind, typefind);
 	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->typefind);
 
-	rb_debug ("constructing autoplugger");
-	decoder_name = "spider";
-
-	mp->priv->decoder = gst_element_factory_make (decoder_name, "autoplugger");
-	if (mp->priv->decoder == NULL) {
-		char *err = g_strdup_printf (_("Failed to create %s element; check your installation"),
-					     decoder_name);
-		g_set_error (error,
-			     RB_PLAYER_ERROR,
-			     RB_PLAYER_ERROR_NO_DEMUX_PLUGIN,
-			     err);
-		g_free (err);
-		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
-		mp->priv->pipeline = NULL;
-		return;
-	}
+	MAKE_ELEMENT_OR_LOSE(spider, decoder);
 	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->decoder);
 
-	/* Volume */
-	mp->priv->volume = gst_element_factory_make ("volume", "volume");
-	if (mp->priv->volume == NULL) {
-		g_set_error (error,
-			     RB_PLAYER_ERROR,
-			     RB_PLAYER_ERROR_NO_VOLUME_PLUGIN,
-			     _("Failed to create volume element; check your installation"));
-		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
-		mp->priv->pipeline = NULL;
-		return;
-	}
+	MAKE_ELEMENT_OR_LOSE(volume, volume);
 	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->volume);
-
 	dpman = gst_dpman_get_manager (mp->priv->volume);
 	gst_dpman_set_mode (dpman, "synchronous");
-	mp->priv->volume_dparam = gst_dpsmooth_new (G_TYPE_FLOAT);
+	mp->priv->volume_dparam = gst_dpsmooth_new (G_TYPE_DOUBLE);
 	g_assert (mp->priv->volume_dparam != NULL);
 	gst_dpman_attach_dparam (dpman, "volume", mp->priv->volume_dparam);
 
+	MAKE_ELEMENT_OR_LOSE(audioconvert, audioconvert);
+	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->audioconvert);
+
+	MAKE_ELEMENT_OR_LOSE(audioscale, audioscale);
+	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->audioscale);
+
 	/* Output sink */
 	mp->priv->sink = gst_gconf_get_default_audio_sink ();
 	if (mp->priv->sink == NULL) {
@@ -574,24 +532,21 @@ rb_player_construct (RBPlayer *mp,
 	}
 	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->sink);
 
-	gst_element_link_many (mp->priv->decoder, mp->priv->volume, mp->priv->sink, NULL);
+	gst_element_link_many (mp->priv->decoder, mp->priv->volume, mp->priv->audioconvert, mp->priv->audioscale, mp->priv->sink, NULL);
 	if (iradio_mode)
 		gst_element_link_many (mp->priv->src, mp->priv->queue, mp->priv->typefind, mp->priv->decoder, NULL);
 	else
 		gst_element_link_many (mp->priv->src, mp->priv->typefind, mp->priv->decoder, NULL);
 
-	g_signal_connect (G_OBJECT (mp->priv->sink), "eos",
-			  G_CALLBACK (eos_cb), mp);
+	g_signal_connect_object (G_OBJECT (mp->priv->sink), "eos",
+				 G_CALLBACK (eos_cb), mp, 0);
 
 	if (mp->priv->cur_volume > 1.0)
 		mp->priv->cur_volume = 1.0;
 	if (mp->priv->cur_volume < 0.0)
 		mp->priv->cur_volume = 0;
 	g_object_set (G_OBJECT (mp->priv->volume_dparam),
-		      "value_float", mp->priv->cur_volume,
-		      NULL);
-	g_object_set (G_OBJECT (mp->priv->volume),
-		      "mute", mp->priv->mute,
+		      "value_double", mp->priv->cur_volume,
 		      NULL);
 
 	if (mp->priv->timer)
@@ -601,12 +556,36 @@ rb_player_construct (RBPlayer *mp,
 	g_timer_reset (mp->priv->timer);
 	mp->priv->timer_add = 0;
 	rb_debug ("pipeline construction complete");
+	return;
+missing_element:
+	{
+		char *err = g_strdup_printf (_("Failed to create %s element; check your installation"),
+					     element_name);
+		g_set_error (error,
+			     RB_PLAYER_ERROR,
+			     RB_PLAYER_ERROR_GENERAL,
+			     err);
+		g_free (err);
+		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
+		mp->priv->pipeline = NULL;
+	}
 }
 
 RBPlayer *
 rb_player_new (GError **error)
 {
 	RBPlayer *mp;
+	GstElement *dummy;
+
+	dummy = gst_element_factory_make ("fakesink", "fakesink");
+	if (!dummy
+	    || !gst_scheduler_factory_make (NULL, GST_ELEMENT (dummy))) {
+		g_set_error (error, RB_PLAYER_ERROR,
+			     RB_PLAYER_ERROR_GENERAL,
+			     "%s",
+			     _("Couldn't initialize scheduler.  Did you run gst-register?"));
+		return NULL;
+	}
 
 	mp = RB_PLAYER (g_object_new (RB_TYPE_PLAYER, NULL));
 
@@ -663,6 +642,7 @@ rb_player_sync_pipeline (RBPlayer *mp, gboolean iradio_mode, GError **error)
 			return FALSE;
 		}
 			
+#if 0
 		rb_debug ("setting sink to NULL");
 		if (gst_element_set_state (mp->priv->sink, GST_STATE_NULL) != GST_STATE_SUCCESS) {
 			g_set_error (error,
@@ -671,14 +651,15 @@ rb_player_sync_pipeline (RBPlayer *mp, gboolean iradio_mode, GError **error)
 				     _("Could not close output sink"));
 			return FALSE;
 		}
+#endif
 	}
 	return TRUE;
 }
 
 void
 rb_player_open (RBPlayer *mp,
-			  const char *uri,
-			  GError **error)
+		const char *uri,
+		GError **error)
 {
 	gboolean audiocd_mode = uri && g_str_has_prefix (uri, "audiocd://");
 	gboolean iradio_mode = uri && g_str_has_prefix (uri, "http://");
@@ -780,12 +761,12 @@ rb_player_close (RBPlayer *mp, GError **error)
 	mp->priv->pipeline = NULL;
 }
 
-const char *
-rb_player_get_uri (RBPlayer *mp)
+gboolean
+rb_player_opened (RBPlayer *mp)
 {
-	g_return_val_if_fail (RB_IS_PLAYER (mp), NULL);
+	g_return_val_if_fail (RB_IS_PLAYER (mp), FALSE);
 
-	return mp->priv->uri;
+	return mp->priv->uri != NULL;
 }
 
 void
@@ -818,8 +799,6 @@ rb_player_pause (RBPlayer *mp)
 
 	gst_element_set_state (mp->priv->pipeline,
 			       GST_STATE_PAUSED);
-	gst_element_set_state (mp->priv->sink,
-			       GST_STATE_NULL);
 }
 
 gboolean
@@ -867,7 +846,7 @@ rb_player_set_replaygain (RBPlayer *mp,
 
 	if (mp->priv->pipeline != NULL) {
 		g_object_set (G_OBJECT (mp->priv->volume_dparam),
-			      "value_float",
+			      "value_double",
 			      mp->priv->cur_volume * scale,
 			      NULL);
 	}
@@ -875,14 +854,14 @@ rb_player_set_replaygain (RBPlayer *mp,
 
 void
 rb_player_set_volume (RBPlayer *mp,
-				float volume)
+		      float volume)
 {
 	g_return_if_fail (RB_IS_PLAYER (mp));
 	g_return_if_fail (volume >= 0.0 && volume <= 1.0);
 
 	if (mp->priv->pipeline != NULL) {
 		g_object_set (G_OBJECT (mp->priv->volume_dparam),
-			      "value_float",
+			      "value_double",
 			      volume,
 			      NULL);
 	}
@@ -898,30 +877,6 @@ rb_player_get_volume (RBPlayer *mp)
 	return mp->priv->cur_volume;
 }
 
-void
-rb_player_set_mute (RBPlayer *mp,
-			      gboolean mute)
-{
-	g_return_if_fail (RB_IS_PLAYER (mp));
-
-	if (mp->priv->pipeline != NULL)
-		g_object_set (G_OBJECT (mp->priv->volume),
-			      "mute",
-			      mute,
-			      NULL);
-	else
-
-	mp->priv->mute = mute;
-}
-
-gboolean
-rb_player_get_mute (RBPlayer *mp)
-{
-	g_return_val_if_fail (RB_IS_PLAYER (mp), FALSE);
-
-	return mp->priv->mute;
-}
-
 gboolean
 rb_player_seekable (RBPlayer *mp)
 {
