@@ -55,6 +55,8 @@
 #include "rb-metadata.h"
 #include "rb-volume.h"
 #include "rb-remote.h"
+#include "rb-iradio-source.h"
+#include "rb-library-source.h"
 #include "eel-gconf-extensions.h"
 
 #include "rb-play-order.h"
@@ -70,6 +72,10 @@ typedef enum
 	PLAY_BUTTON_STOP
 } PlayButtonState;
 
+static const char* const state_to_play_order[2][2] =
+	{{"linear",	"linear-loop"},
+	 {"shuffle",	"random-by-age-and-rating"}};
+
 static void rb_shell_player_class_init (RBShellPlayerClass *klass);
 static void rb_shell_player_init (RBShellPlayer *shell_player);
 static GObject *rb_shell_player_constructor (GType type, guint n_construct_properties,
@@ -151,10 +157,16 @@ static void info_available_cb (RBPlayer *player,
 			       gpointer data);
 static void buffering_end_cb (RBPlayer *player, gpointer data);
 static void buffering_begin_cb (RBPlayer *player, gpointer data);
+static void buffering_progress_cb (RBPlayer *player, int progress, gpointer data);
+static void rb_shell_player_enable_buffering (RBShellPlayer *player);
 static void rb_shell_player_disable_buffering (RBShellPlayer *player);
 
-static void rb_shell_player_sync_play_order (RBShellPlayer *player);
+static void rb_shell_player_set_play_order (RBShellPlayer *player,
+					    const gchar *new_val);
 
+static void rb_shell_player_sync_play_order (RBShellPlayer *player);
+static void gconf_play_order_changed (GConfClient *client,guint cnxn_id,
+				      GConfEntry *entry, RBShellPlayer *player);
 
 #ifdef HAVE_MMKEYS
 static void grab_mmkey (int key_code, GdkWindow *root);
@@ -235,6 +247,9 @@ struct RBShellPlayerPrivate
 	GtkWidget *magic_button;
 
 	RBRemote *remote;
+
+	guint gconf_play_order_id;
+	guint gconf_state_id;
 };
 
 enum
@@ -243,7 +258,6 @@ enum
 	PROP_SOURCE,
 	PROP_COMPONENT,
 	PROP_TRAY_COMPONENT,
-	PROP_REPEAT,
 	PROP_PLAY_ORDER,
 	PROP_PLAYING,
 	PROP_BUFFERING,
@@ -279,19 +293,6 @@ static GObjectClass *parent_class = NULL;
 
 static guint rb_shell_player_signals[LAST_SIGNAL] = { 0 };
 
-static void
-gconf_key_changed (GConfClient *client,guint cnxn_id,
-		   GConfEntry *entry, RBShellPlayer *player)
-{
-	if (strcmp (CONF_STATE_REPEAT, entry->key) == 0) {
-		g_object_notify (G_OBJECT (player), "repeat");
-	} else if (strcmp (CONF_STATE_PLAY_ORDER, entry->key) == 0) {
-		rb_shell_player_sync_play_order (player);
-		g_object_notify (G_OBJECT (player), "play-order");
-	}
-}
-
-
 GType
 rb_shell_player_get_type (void)
 {
@@ -320,37 +321,6 @@ rb_shell_player_get_type (void)
 	return rb_shell_player_type;
 }
 
-void 
-rb_shell_player_set_repeat (RBShellPlayer *player, gboolean new_val)
-{
-	gboolean old_val;
-	/* Warning: there's probably a small race if the value stored in 
-	 * gconf is modified after the old_val != new_val test, but that
-	 * doesn't matter that much
-	 */
-	g_object_get (G_OBJECT (player), "repeat", &old_val, NULL);
-	if (old_val != new_val) {
-		/* The notify signal will be emitted by the gconf notifier */
-		eel_gconf_set_boolean (CONF_STATE_REPEAT, new_val);
-	}
-}
-
-
-void 
-rb_shell_player_set_play_order (RBShellPlayer *player, const gchar *new_val)
-{
-	const gchar *old_val;
-	/* Warning: there's probably a small race if the value stored in 
-	 * gconf is modified after the old_val != new_val test, but that
-	 * doesn't matter that much
-	 */
-	g_object_get (G_OBJECT (player), "play-order", &old_val, NULL);
-	if (strcmp (old_val, new_val) != 0) {
-		/* The notify signal will be emitted by the gconf notifier */
-		eel_gconf_set_string (CONF_STATE_PLAY_ORDER, new_val);
-	}
-}
-
 static void
 rb_shell_player_class_init (RBShellPlayerClass *klass)
 {
@@ -389,13 +359,6 @@ rb_shell_player_class_init (RBShellPlayerClass *klass)
 
 	/* If you change these, be sure to update the CORBA interface
 	 * in rb-shell.c! */
-	g_object_class_install_property (object_class,
-					 PROP_REPEAT,
-					 g_param_spec_boolean ("repeat", 
-							       "repeat", 
-							       "Whether repeat is enabled or not", 
-							       FALSE,
-							       G_PARAM_READABLE));
 	g_object_class_install_property (object_class,
 					 PROP_PLAY_ORDER,
 					 g_param_spec_string ("play-order", 
@@ -487,42 +450,45 @@ rb_shell_player_init (RBShellPlayer *player)
 
 	gtk_box_set_spacing (GTK_BOX (player), 12);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "info",
-			  G_CALLBACK (info_available_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "info",
+				 G_CALLBACK (info_available_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "eos",
-			  G_CALLBACK (eos_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "eos",
+				 G_CALLBACK (eos_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "tick",
-			  G_CALLBACK (tick_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "tick",
+				 G_CALLBACK (tick_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "error",
-			  G_CALLBACK (error_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "error",
+				 G_CALLBACK (error_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "buffering_begin",
-			  G_CALLBACK (buffering_begin_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "buffering_begin",
+				 G_CALLBACK (buffering_begin_cb),
+				 player, 0);
 
-	g_signal_connect (G_OBJECT (player->priv->mmplayer),
-			  "buffering_end",
-			  G_CALLBACK (buffering_end_cb),
-			  player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "buffering_end",
+				 G_CALLBACK (buffering_end_cb),
+				 player, 0);
 
-	eel_gconf_notification_add (CONF_STATE_REPEAT,
-				    (GConfClientNotifyFunc)gconf_key_changed,
-				    player);
-	eel_gconf_notification_add (CONF_STATE_PLAY_ORDER,
-				    (GConfClientNotifyFunc)gconf_key_changed,
-				    player);
+	g_signal_connect_object (G_OBJECT (player->priv->mmplayer),
+				 "buffering_progress",
+				 G_CALLBACK (buffering_progress_cb),
+				 player, 0);
+
+	player->priv->gconf_play_order_id =
+		eel_gconf_notification_add (CONF_STATE_PLAY_ORDER,
+					    (GConfClientNotifyFunc)gconf_play_order_changed,
+					    player);
 
 	rb_shell_player_sync_volume (player);
 
@@ -595,9 +561,10 @@ rb_shell_player_init (RBShellPlayer *player)
 	gtk_container_add (GTK_CONTAINER (alignment), player->priv->volume_button);
 	gtk_box_pack_end (GTK_BOX (player), alignment, FALSE, TRUE, 0);
 
-	eel_gconf_notification_add (CONF_STATE,
-				    (GConfClientNotifyFunc) rb_shell_player_state_changed_cb,
-				    player);
+	player->priv->gconf_state_id = 
+		eel_gconf_notification_add (CONF_STATE,
+					    (GConfClientNotifyFunc) rb_shell_player_state_changed_cb,
+					    player);
 
 #ifdef HAVE_MMKEYS
 	/* Enable Multimedia Keys */
@@ -617,6 +584,9 @@ rb_shell_player_finalize (GObject *object)
 
 	g_return_if_fail (player->priv != NULL);
 
+	eel_gconf_notification_remove(player->priv->gconf_play_order_id);
+	eel_gconf_notification_remove(player->priv->gconf_state_id);
+
 	eel_gconf_set_float (CONF_STATE_VOLUME,
 			     rb_player_get_volume (player->priv->mmplayer));
 
@@ -674,19 +644,19 @@ rb_shell_player_set_property (GObject *object,
 			RBEntryView *songs = rb_source_get_entry_view (player->priv->selected_source);
 			GList *extra_views = rb_source_get_extra_views (player->priv->selected_source);
 
-			g_signal_connect (G_OBJECT (songs),
-					  "changed",
-					  G_CALLBACK (rb_shell_player_entry_view_changed_cb),
-					  player);
-			g_signal_connect (G_OBJECT (songs),
-					  "entry-activated",
-					  G_CALLBACK (rb_shell_player_entry_activated_cb),
-					  player);
+			g_signal_connect_object (G_OBJECT (songs),
+						 "changed",
+						 G_CALLBACK (rb_shell_player_entry_view_changed_cb),
+						 player, 0);
+			g_signal_connect_object (G_OBJECT (songs),
+						 "entry-activated",
+						 G_CALLBACK (rb_shell_player_entry_activated_cb),
+						 player, 0);
 			for (; extra_views; extra_views = extra_views->next)
-				g_signal_connect (G_OBJECT (extra_views->data),
-						  "property-activated",
-						  G_CALLBACK (rb_shell_player_property_row_activated_cb),
-						  player);
+				g_signal_connect_object (G_OBJECT (extra_views->data),
+							 "property-activated",
+							 G_CALLBACK (rb_shell_player_property_row_activated_cb),
+							 player, 0);
 
 			g_list_free (extra_views);
 			
@@ -714,10 +684,6 @@ rb_shell_player_set_property (GObject *object,
 							     rb_shell_player_verbs,
 							     player);
 		break;
-	case PROP_REPEAT:
-		eel_gconf_set_boolean (CONF_STATE_REPEAT, 
-				       g_value_get_boolean (value));
-		break;
 
 	case PROP_PLAY_ORDER:
 		eel_gconf_set_string (CONF_STATE_PLAY_ORDER, 
@@ -749,9 +715,6 @@ rb_shell_player_get_property (GObject *object,
 	case PROP_TRAY_COMPONENT:
 		g_value_set_object (value, player->priv->tray_component);
 		break;
-	case PROP_REPEAT:
-		g_value_set_boolean (value, eel_gconf_get_boolean (CONF_STATE_REPEAT));
-		break;
 	case PROP_PLAY_ORDER:
 		g_value_set_string_take_ownership (value, eel_gconf_get_string (CONF_STATE_PLAY_ORDER));
 		break;
@@ -982,6 +945,56 @@ rb_shell_player_set_playing_entry (RBShellPlayer *player, RhythmDBEntry *entry)
 	rb_shell_player_sync_replaygain (player);
 }
 
+static void
+gconf_play_order_changed (GConfClient *client,guint cnxn_id,
+			  GConfEntry *entry, RBShellPlayer *player)
+{
+	rb_shell_player_sync_play_order (player);
+	g_object_notify (G_OBJECT (player), "play-order");
+}
+
+gboolean
+rb_shell_player_get_playback_state (RBShellPlayer *player,
+				    gboolean *shuffle,
+				    gboolean *repeat)
+{
+	int i, j;
+	char *play_order;
+
+	play_order = eel_gconf_get_string (CONF_STATE_PLAY_ORDER);
+
+	for (i = 0; i < G_N_ELEMENTS(state_to_play_order); i++)
+		for (j = 0; j < G_N_ELEMENTS(state_to_play_order[0]); j++)
+			if (!strcmp (play_order, state_to_play_order[i][j]))
+				goto found;
+
+	return FALSE;
+
+found:
+	*shuffle = i > 0;
+	*repeat = j > 0;
+	return TRUE;
+}
+
+static void 
+rb_shell_player_set_play_order (RBShellPlayer *player, const gchar *new_val)
+{
+	char *old_val;
+	g_object_get (G_OBJECT (player), "play-order", &old_val, NULL);
+	if (strcmp (old_val, new_val) != 0) {
+		/* The notify signal will be emitted by the gconf notifier */
+		eel_gconf_set_string (CONF_STATE_PLAY_ORDER, new_val);
+	}
+	g_free (old_val);
+}
+
+void
+rb_shell_player_set_playback_state (RBShellPlayer *player, gboolean shuffle, gboolean repeat)
+{
+	const char *neworder = state_to_play_order[shuffle ? 1 : 0][repeat ? 1 : 0];
+	rb_shell_player_set_play_order (player, neworder);
+}
+
 static void
 rb_shell_player_sync_play_order (RBShellPlayer *player)
 {
@@ -1033,25 +1046,43 @@ rb_shell_player_do_previous (RBShellPlayer *player)
 	RhythmDBEntry* entry;
 
 	if (player->priv->source != NULL) {
-		/* If we're in the first 2 seconds go to the previous song,
-		 * else restart the current one.
-		 */
-		if (rb_player_get_time (player->priv->mmplayer) < 3) {
-			rb_debug ("doing previous");
-			entry = rb_play_order_get_previous (player->priv->play_order);
-			if (entry) {
-				rb_play_order_go_previous (player->priv->play_order);
-				rb_shell_player_set_playing_entry (player, entry);
+
+		if (RB_IS_LIBRARY_SOURCE (player->priv->source)) {
+
+			rb_debug("previous algorithm for a library source");
+			
+			/* If we're in the first 2 seconds go to the previous song,
+			 * else restart the current one.
+			 */
+			if (rb_player_get_time (player->priv->mmplayer) < 3) {
+				rb_debug ("doing previous");
+				entry = rb_play_order_get_previous (player->priv->play_order);
+				if (entry) {
+					rb_play_order_go_previous (player->priv->play_order);
+					rb_shell_player_set_playing_entry (player, entry);
+				} else {
+					/* Is this the right thing to do when there's no previous song? */
+					rb_debug ("No previous entry, restarting song");
+					rb_player_set_time (player->priv->mmplayer, 0);
+					rb_header_sync_time (player->priv->header_widget);
+				}
 			} else {
-				/* Is this the right thing to do when there's no previous song? */
-				rb_debug ("No previous entry, restarting song");
+				rb_debug ("restarting song");
 				rb_player_set_time (player->priv->mmplayer, 0);
 				rb_header_sync_time (player->priv->header_widget);
 			}
-		} else {
-			rb_debug ("restarting song");
-			rb_player_set_time (player->priv->mmplayer, 0);
-			rb_header_sync_time (player->priv->header_widget);
+		}
+		else if (RB_IS_IRADIO_SOURCE (player->priv->source)) {
+		
+			rb_debug("previous algorithm for an iradio source");
+			
+			entry = rb_play_order_get_previous (player->priv->play_order);
+			
+			if (entry) {
+				rb_debug ("previous station found, doing previous");
+				rb_play_order_go_previous (player->priv->play_order);
+				rb_shell_player_set_playing_entry (player, entry);
+			}
 		}
 
 		rb_shell_player_jump_to_current (player);
@@ -1297,17 +1328,17 @@ rb_shell_player_cmd_stop (BonoboUIComponent *component,
 static void
 rb_shell_player_sync_control_state (RBShellPlayer *player)
 {
-	char *play_order;
-	
+	gboolean shuffle, repeat;
 	rb_debug ("syncing control state");
-	play_order = eel_gconf_get_string (CONF_STATE_PLAY_ORDER);
+
+	if (!rb_shell_player_get_playback_state (player, &shuffle,
+						 &repeat))
+		return;
+	
 	rb_bonobo_set_active (player->priv->component,
-			      CMD_PATH_SHUFFLE,
-			      strcmp (play_order, "linear") != 0);
-	g_free (play_order);
+			      CMD_PATH_SHUFFLE, shuffle);
 	rb_bonobo_set_active (player->priv->component,
-			      CMD_PATH_REPEAT,
-			      eel_gconf_get_boolean (CONF_STATE_REPEAT));
+			      CMD_PATH_REPEAT, repeat);
 }
 
 static void
@@ -1371,26 +1402,33 @@ rb_shell_player_shuffle_changed_cb (BonoboUIComponent *component,
 				    const char *state,
 				    RBShellPlayer *player)
 {
-	gboolean newval = rb_bonobo_get_active (component, CMD_PATH_SHUFFLE);
+	const char *neworder;
+	gboolean shuffle, repeat;
 	rb_debug ("shuffle changed");
 
-	if (newval) {
-		rb_shell_player_set_play_order (player, "shuffle");
-	} else {
-		rb_shell_player_set_play_order (player, "linear");
-	}
+	if (!rb_shell_player_get_playback_state (player, &shuffle, &repeat))
+		return;
+	shuffle = !shuffle;
+	neworder = state_to_play_order[shuffle ? 1 : 0][repeat ? 1 : 0];
+	rb_shell_player_set_play_order (player, neworder);
 }
 	
-static void rb_shell_player_repeat_changed_cb (BonoboUIComponent *component,
-					       const char *path,
-					       Bonobo_UIComponent_EventType type,
-					       const char *state,
-					       RBShellPlayer *player)
+static void
+rb_shell_player_repeat_changed_cb (BonoboUIComponent *component,
+				   const char *path,
+				   Bonobo_UIComponent_EventType type,
+				   const char *state,
+				   RBShellPlayer *player)
 {
-	gboolean newval = rb_bonobo_get_active (component, CMD_PATH_REPEAT);
+	const char *neworder;
+	gboolean shuffle, repeat;
 	rb_debug ("repeat changed");
 
-	rb_shell_player_set_repeat (player, newval);
+	if (!rb_shell_player_get_playback_state (player, &shuffle, &repeat))
+		return;
+	repeat = !repeat;
+	neworder = state_to_play_order[shuffle ? 1 : 0][repeat ? 1 : 0];
+	rb_shell_player_set_play_order (player, neworder);
 }
 
 static void
@@ -1582,6 +1620,7 @@ rb_shell_player_sync_with_source (RBShellPlayer *player)
 	else
 		rb_header_set_title (player->priv->header_widget, entry_title);
 	g_free (title);
+	
 	rb_header_set_playing_entry (player->priv->header_widget, entry);
 	rb_header_sync (player->priv->header_widget);
 }
@@ -1698,10 +1737,10 @@ rb_shell_player_set_playing_source_internal (RBShellPlayer *player,
 
 	if (source != NULL) {
 		RBEntryView *songs = rb_source_get_entry_view (player->priv->source);
-		g_signal_connect (G_OBJECT (songs),
-				  "playing_entry_deleted",
-				  G_CALLBACK (rb_shell_player_playing_entry_deleted_cb),
-				  player);
+		g_signal_connect_object (G_OBJECT (songs),
+					 "playing_entry_deleted",
+					 G_CALLBACK (rb_shell_player_playing_entry_deleted_cb),
+					 player, 0);
 	}
 
 	if (player->priv->play_order)
@@ -2002,23 +2041,34 @@ info_available_cb (RBPlayer *mmplayer,
 }
 
 static void
-buffering_begin_cb (RBPlayer *mmplayer,
-		    gpointer data)
+rb_shell_player_enable_buffering (RBShellPlayer *player)
 {
-	RBShellPlayer *player = RB_SHELL_PLAYER (data);
-	player->priv->buffering = TRUE;
-	g_object_notify (G_OBJECT (player), "buffering");
+	if (!player->priv->buffering) {
+		rb_debug("enabling buffering");
+		player->priv->buffering = TRUE;
+		g_object_notify (G_OBJECT (player), "buffering");
+	}
 }
 
 static void
 rb_shell_player_disable_buffering (RBShellPlayer *player)
 {
 	if (player->priv->buffering) {
+		rb_debug("disabling buffering");
 		player->priv->buffering = FALSE;
 		g_object_notify (G_OBJECT (player), "buffering");
 	}
 }
 
+static void
+buffering_begin_cb (RBPlayer *mmplayer,
+		    gpointer data)
+{
+	RBShellPlayer *player = RB_SHELL_PLAYER (data);
+	rb_debug ("got buffering_begin_cb");
+	rb_shell_player_enable_buffering (player);
+}
+
 static void
 buffering_end_cb (RBPlayer *mmplayer,
 		  gpointer data)
@@ -2030,6 +2080,30 @@ buffering_end_cb (RBPlayer *mmplayer,
  		rb_source_buffering_done (player->priv->source);
 }
 
+static void
+buffering_progress_cb (RBPlayer *mmplayer,
+		       int progress,
+		       gpointer data)
+{
+	RBShellPlayer *player = RB_SHELL_PLAYER (data);
+	rb_debug ("got buffering_progress_cb: %d", progress);
+
+	GDK_THREADS_ENTER();
+
+	if (progress == 100)
+	{
+		rb_shell_player_disable_buffering (player);
+		rb_shell_player_sync_with_source (player);
+		rb_shell_player_sync_buttons (player);
+	}	
+	else
+	{
+		rb_shell_player_enable_buffering (player);
+	}
+	
+	GDK_THREADS_LEAVE();
+}
+
 const char *
 rb_shell_player_get_playing_path (RBShellPlayer *shell_player)
 {
