@@ -68,6 +68,9 @@
 #include "rb-playlist.h"
 #include "rb-bonobo-helpers.h"
 #include "rb-library-source.h"
+#ifdef WITH_IPOD_SUPPORT
+#include "rb-ipod-source.h"
+#endif /* WITH_IPOD_SUPPORT */
 #include "rb-load-failure-dialog.h"
 #include "rb-new-station-dialog.h"
 #include "rb-iradio-source.h"
@@ -144,7 +147,10 @@ static void paned_size_allocate_cb (GtkWidget *widget,
 				    GtkAllocation *allocation,
 				    RBShell *shell);
 static void rb_shell_select_source (RBShell *shell, RBSource *source);
+static void rb_shell_select_source_internal (RBShell *shell, RBSource *source);
 static void rb_shell_append_source (RBShell *shell, RBSource *source);
+static RBSource *rb_shell_get_source_by_entry_type (RBShell *shell, 
+						    RhythmDBEntryType type);
 static void source_selected_cb (RBSourceList *sourcelist,
 				RBSource *source,
 				RBShell *shell);
@@ -228,6 +234,12 @@ static void paned_changed_cb (GConfClient *client,
 			      guint cnxn_id,
 			      GConfEntry *entry,
 			      RBShell *shell);
+static void rb_shell_sync_selected_source (RBShell *shell);
+
+static void selected_source_changed_cb (GConfClient *client,
+					guint cnxn_id,
+					GConfEntry *entry,
+					RBShell *shell);
 #ifdef HAVE_AUDIOCD
 static void audiocd_changed_cb (MonkeyMediaAudioCD *cd,
 				gboolean available,
@@ -242,6 +254,19 @@ static gboolean rb_shell_show_popup_cb (RBSourceList *sourcelist,
 					RBShell *shell);
 static void tray_deleted_cb (GtkWidget *win, GdkEventAny *event, RBShell *shell);
 
+typedef RBSource *(*SourceCreateFunc)(RBShell *, RhythmDB *db, 
+				      BonoboUIComponent *component);
+static SourceCreateFunc known_sources[] = { 
+	rb_library_source_new,
+	rb_iradio_source_new,
+#ifdef WITH_IPOD_SUPPORT
+	rb_ipod_source_new,	
+#endif /* WITH_IPOD_SUPPORT */
+	NULL,
+};
+
+
+
 static gboolean save_yourself_cb (GnomeClient *client, 
                                   gint phase,
                                   GnomeSaveStyle save_style,
@@ -266,8 +291,10 @@ enum
 	PROP_ARGC,
 	PROP_ARGV,
 	PROP_NO_REGISTRATION,
+	PROP_NO_UPDATE,
 	PROP_DRY_RUN,
 	PROP_RHYTHMDB_FILE,
+	PROP_SELECTED_SOURCE
 };
 
 #define CMD_PATH_VIEW_SMALLDISPLAY "/commands/ToggleSmallDisplay"
@@ -282,6 +309,7 @@ enum
 #define CONF_STATE_WINDOW_MAXIMIZED CONF_PREFIX "/state/window_maximized"
 #define CONF_STATE_PANED_POSITION   CONF_PREFIX "/state/paned_position"
 #define CONF_STATE_ADD_DIR          CONF_PREFIX "/state/add_dir"
+#define CONF_STATE_SELECTED_SOURCE  CONF_PREFIX "/state/selected_source"
 
 struct RBShellPrivate
 {
@@ -296,15 +324,20 @@ struct RBShellPrivate
 	GtkWidget *hsep;
 
 	GList *sources;
+	GHashTable *sources_hash;
 
 	gboolean play_queued;
 
 	gboolean db_dirty;
 	guint async_state_save_id;
 
+	gboolean shutting_down;
+	gboolean load_complete;
+
 	int argc;
 	char **argv;
 	gboolean no_registration;
+	gboolean no_update;
 	gboolean dry_run;
 	char *rhythmdb_file;
 
@@ -316,13 +349,10 @@ struct RBShellPrivate
 	RBStatusbar *statusbar;
 	RBPlaylistManager *playlist_manager;
 
-	RBSource *library_source;
 	GtkWidget *load_error_dialog;
 	GList *supported_media_extensions;
 	gboolean show_db_errors;
 
- 	RBIRadioSource *iradio_source;
-
 #ifdef HAVE_AUDIOCD
  	MonkeyMediaAudioCD *cd;
 #endif
@@ -441,6 +471,14 @@ rb_shell_class_init (RBShellClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property (object_class,
+					 PROP_NO_UPDATE,
+					 g_param_spec_boolean ("no-update", 
+							       "no-update", 
+							       "Whether or not to update the library", 
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	g_object_class_install_property (object_class,
 					 PROP_DRY_RUN,
 					 g_param_spec_boolean ("dry-run", 
@@ -456,6 +494,14 @@ rb_shell_class_init (RBShellClass *klass)
 							      "The RhythmDB file to use", 
 							      "rhythmdb.xml",
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_SELECTED_SOURCE,
+					 g_param_spec_pointer ("selected-source", 
+							       "selected-source", 
+							       "Source which is currently selected", 
+							       G_PARAM_READABLE));
+
 }
 
 static void
@@ -470,9 +516,11 @@ rb_shell_init (RBShell *shell)
 	rb_ensure_dir_exists (rb_dot_dir ());
 
 	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, "rhythmbox.png", TRUE, NULL);
-	gnome_window_icon_set_default_from_file (file);
-	g_free (file);
-	
+	if (file) {
+		gnome_window_icon_set_default_from_file (file);
+		g_free (file);
+	}
+
         rb_shell_session_init (shell);
 
 	eel_gconf_monitor_add (CONF_PREFIX);
@@ -498,6 +546,9 @@ rb_shell_set_property (GObject *object,
 	case PROP_NO_REGISTRATION:
 		shell->priv->no_registration = g_value_get_boolean (value);
 		break;
+	case PROP_NO_UPDATE:
+		shell->priv->no_update = g_value_get_boolean (value);
+		break;
 	case PROP_DRY_RUN:
 		shell->priv->dry_run = g_value_get_boolean (value);
 		if (shell->priv->dry_run)
@@ -531,12 +582,18 @@ rb_shell_get_property (GObject *object,
 	case PROP_NO_REGISTRATION:
 		g_value_set_boolean (value, shell->priv->no_registration);
 		break;
+	case PROP_NO_UPDATE:
+		g_value_set_boolean (value, shell->priv->no_update);
+		break;
 	case PROP_DRY_RUN:
 		g_value_set_boolean (value, shell->priv->dry_run);
 		break;
 	case PROP_RHYTHMDB_FILE:
 		g_value_set_string (value, shell->priv->rhythmdb_file);
 		break;
+	case PROP_SELECTED_SOURCE:
+		g_value_set_pointer (value, shell->priv->selected_source);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -546,15 +603,22 @@ rb_shell_get_property (GObject *object,
 static gboolean
 rb_shell_sync_state (RBShell *shell)
 {
-	if (shell->priv->dry_run)
+	if (shell->priv->dry_run) {
+		rb_debug ("in dry-run mode, not syncing state");
 		return FALSE;
+	}
+
+	if (!shell->priv->load_complete) {
+		rb_debug ("load incomplete, not syncing state");
+		return FALSE;
+	}
 	
 	rb_debug ("saving playlists");
-	rb_playlist_manager_save_playlists (shell->priv->playlist_manager);
+	rb_playlist_manager_save_playlists_blocking (shell->priv->playlist_manager);
 
 	rb_debug ("saving db");
 	rhythmdb_read_lock (shell->priv->db);
-	rhythmdb_save (shell->priv->db);
+	rhythmdb_save_blocking (shell->priv->db);
 	rhythmdb_read_unlock (shell->priv->db);
 	return FALSE;
 }
@@ -570,6 +634,24 @@ idle_save_state (RBShell *shell)
 	return FALSE;
 }
 
+static gboolean
+idle_save_rhythmdb (RhythmDB *db)
+{
+	rhythmdb_read_lock (db);
+	rhythmdb_save (db);
+	rhythmdb_read_unlock (db);
+	
+	return FALSE;
+}
+
+static gboolean
+idle_save_playlist_manager (RBPlaylistManager *mgr)
+{
+	rb_playlist_manager_save_playlists (mgr);
+
+	return TRUE;
+}
+
 static void
 rb_shell_db_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
 			RBShell *shell)
@@ -585,6 +667,21 @@ rb_shell_db_changed_prop_cb (RhythmDB *db, RhythmDBEntry *entry,
 	rb_shell_db_changed_cb (db, entry, shell);
 }
 
+static void
+rb_shell_shutdown (RBShell *shell)
+{
+	if (shell->priv->shutting_down)
+		return;
+	shell->priv->shutting_down = TRUE;
+
+	rb_debug ("Unregistering with bonobo");
+	bonobo_activation_active_server_unregister (RB_SHELL_OAFIID, bonobo_object_corba_objref (BONOBO_OBJECT (shell)));
+	
+	bonobo_activation_active_server_unregister (RB_FACTORY_OAFIID, bonobo_object_corba_objref (BONOBO_OBJECT (shell)));
+	rb_debug ("Unregistered with Bonobo Activation");
+	
+}
+
 static void
 rb_shell_finalize (GObject *object)
 {
@@ -596,19 +693,13 @@ rb_shell_finalize (GObject *object)
 
 	eel_gconf_monitor_remove (CONF_PREFIX);
 
-	bonobo_activation_active_server_unregister (RB_SHELL_OAFIID, bonobo_object_corba_objref (BONOBO_OBJECT (shell)));
-	
-	bonobo_activation_active_server_unregister (RB_FACTORY_OAFIID, bonobo_object_corba_objref (BONOBO_OBJECT (shell)));
-
-
-	rb_debug ("Unregistered with Bonobo Activation");
-	
 	gtk_widget_destroy (GTK_WIDGET (shell->priv->load_error_dialog));
 	g_list_free (shell->priv->supported_media_extensions);
 
 	gtk_widget_destroy (GTK_WIDGET (shell->priv->tray_icon));
 	
 	g_list_free (shell->priv->sources);
+	g_hash_table_destroy (shell->priv->sources_hash);
 
 	g_object_unref (G_OBJECT (shell->priv->playlist_manager));
 
@@ -637,13 +728,15 @@ rb_shell_finalize (GObject *object)
 }
 
 RBShell *
-rb_shell_new (int argc, char **argv, gboolean no_registration, gboolean dry_run,
+rb_shell_new (int argc, char **argv, gboolean no_registration,
+	      gboolean no_update, gboolean dry_run,
 	      char *rhythmdb)
 {
 	RBShell *s;
 
 	s = g_object_new (RB_TYPE_SHELL, "argc", argc, "argv", argv,
 			  "no-registration", no_registration,
+			  "no-update", no_update,
 			  "dry-run", dry_run, "rhythmdb-file", rhythmdb, NULL);
 
 	return s;
@@ -891,8 +984,9 @@ shell_pb_get_prop (BonoboPropertyBag *bag,
 
 	case PROP_SONG: {
 		GNOME_Rhythmbox_SongInfo *ret_val;
+		
 		ret_val = get_song_info_from_player (shell);
-		(GNOME_Rhythmbox_SongInfo*)arg->_value = ret_val;
+		arg->_value = (gpointer)ret_val;
 		if (ret_val == NULL) {
 			arg->_type = TC_null;
 		} else {
@@ -914,6 +1008,11 @@ shell_pb_set_prop (BonoboPropertyBag *bag,
 		   CORBA_Environment *ev,
 		   gpointer           user_data)
 {
+	RBShell *shell = RB_SHELL (user_data);
+	RBShellPlayer *player;
+
+	player = RB_SHELL_PLAYER (shell->priv->player_shell);
+	
 	switch (arg_id) {
 
 	case PROP_VISIBILITY:
@@ -923,6 +1022,22 @@ shell_pb_set_prop (BonoboPropertyBag *bag,
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_ReadOnly);
 		break;
 
+	case PROP_SHUFFLE:
+	{
+		gboolean repeat;
+		gboolean shuffle;
+
+
+		rb_shell_player_get_playback_state (player, &shuffle,
+						    &repeat);
+		shuffle = BONOBO_ARG_GET_BOOLEAN (arg);
+
+		rb_shell_player_set_playback_state (player,
+						    shuffle,
+						    repeat);
+		break;
+	}
+
 	default:
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
 		break;
@@ -1070,7 +1185,7 @@ rb_shell_entry_changed_cb (GObject *object, GParamSpec *pspec, RBShell *shell)
 	g_assert (strcmp (pspec->name, "playing-entry") == 0);
 	song_info = get_song_info_from_player (shell);
 	arg = bonobo_arg_new (TC_GNOME_Rhythmbox_SongInfo);
-	(GNOME_Rhythmbox_SongInfo*)arg->_value = song_info;
+	arg->_value = (gpointer)song_info;
 	shell_notify_pb_changes (shell, "song", arg);
 	/* FIXME: arg should be released somehow */
 
@@ -1102,7 +1217,10 @@ rb_shell_construct (RBShell *shell)
 	Bonobo_UIContainer corba_container;
 	GtkWidget *vbox;
 	gboolean rhythmdb_exists;
-	
+	RBSource *iradio_source;
+	RBSource *library_source;
+	int i = 0;
+
 	g_return_if_fail (RB_IS_SHELL (shell));
 
 	rb_debug ("Constructing shell");
@@ -1113,15 +1231,15 @@ rb_shell_construct (RBShell *shell)
 
 	shell->priv->window = GTK_WIDGET (win);
 
-	g_signal_connect (G_OBJECT (win), "window-state-event",
-			  G_CALLBACK (rb_shell_window_state_cb),
-			  shell);
-	g_signal_connect (G_OBJECT (win), "configure-event",
-			  G_CALLBACK (rb_shell_window_state_cb),
-			  shell);
-	g_signal_connect (G_OBJECT (win), "delete_event",
-			  G_CALLBACK (rb_shell_window_delete_cb),
-			  shell);
+	g_signal_connect_object (G_OBJECT (win), "window-state-event",
+				 G_CALLBACK (rb_shell_window_state_cb),
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (win), "configure-event",
+				 G_CALLBACK (rb_shell_window_state_cb),
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (win), "delete_event",
+				 G_CALLBACK (rb_shell_window_delete_cb),
+				 shell, 0);
   
 	rb_debug ("shell: creating container area");
 	shell->priv->container = bonobo_window_get_ui_container (win);
@@ -1162,12 +1280,15 @@ rb_shell_construct (RBShell *shell)
 
 		if (shell->priv->dry_run)
 			g_object_set (G_OBJECT (shell->priv->db), "dry-run", TRUE, NULL);
+		if (shell->priv->no_update)
+			g_object_set (G_OBJECT (shell->priv->db), "no-update", TRUE, NULL);
 
 		if (rhythmdb_exists) {
 			g_signal_connect_object (G_OBJECT (shell->priv->db), "load-complete",
 						 G_CALLBACK (rb_shell_load_complete_cb), shell,
 						 0);
-			rhythmdb_load (shell->priv->db);
+		} else {
+			shell->priv->load_complete = TRUE;
 		}
 	}
 	g_signal_connect_object (G_OBJECT (shell->priv->db),
@@ -1204,26 +1325,26 @@ rb_shell_construct (RBShell *shell)
 		shell->priv->player_shell = rb_shell_player_new (shell->priv->ui_component,
 								 tray_component);
 	}
-	g_signal_connect (G_OBJECT (shell->priv->player_shell), 
-			  "notify::repeat", 
-			  G_CALLBACK (rb_shell_property_changed_generic_cb), 
-			  shell);
-	g_signal_connect (G_OBJECT (shell->priv->player_shell), 
-			  "notify::shuffle", 
-			  G_CALLBACK (rb_shell_property_changed_generic_cb), 
-			  shell);
-	g_signal_connect (G_OBJECT (shell->priv->player_shell), 
-			  "notify::playing", 
-			  G_CALLBACK (rb_shell_property_changed_generic_cb), 
-			  shell);
-	g_signal_connect (G_OBJECT (shell->priv->player_shell),
-			  "window_title_changed",
-			  G_CALLBACK (rb_shell_player_window_title_changed_cb),
-			  shell);
-	g_signal_connect (G_OBJECT (shell->priv->player_shell),
-			  "duration_changed",
-			  G_CALLBACK (rb_shell_player_duration_changed_cb),
-			  shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell), 
+				 "notify::repeat", 
+				 G_CALLBACK (rb_shell_property_changed_generic_cb), 
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell), 
+				 "notify::shuffle", 
+				 G_CALLBACK (rb_shell_property_changed_generic_cb), 
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell), 
+				 "notify::playing", 
+				 G_CALLBACK (rb_shell_property_changed_generic_cb), 
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell),
+				 "window_title_changed",
+				 G_CALLBACK (rb_shell_player_window_title_changed_cb),
+				 shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->player_shell),
+				 "duration_changed",
+				 G_CALLBACK (rb_shell_player_duration_changed_cb),
+				 shell, 0);
 	shell->priv->clipboard_shell = rb_shell_clipboard_new (shell->priv->ui_component,
 							       shell->priv->db);
 	shell->priv->source_header = rb_source_header_new (shell->priv->ui_component);
@@ -1231,12 +1352,12 @@ rb_shell_construct (RBShell *shell)
 	shell->priv->paned = gtk_hpaned_new ();
 
 	shell->priv->sourcelist = rb_sourcelist_new ();
-	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "drop_received",
-			  G_CALLBACK (sourcelist_drag_received_cb), shell);
-	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "source_activated",
-			  G_CALLBACK (source_activated_cb), shell);
-	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "show_popup",
-			  G_CALLBACK (rb_shell_show_popup_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->sourcelist), "drop_received",
+				 G_CALLBACK (sourcelist_drag_received_cb), shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->sourcelist), "source_activated",
+				 G_CALLBACK (source_activated_cb), shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->sourcelist), "show_popup",
+				 G_CALLBACK (rb_shell_show_popup_cb), shell, 0);
 
 	shell->priv->statusbar = rb_statusbar_new (shell->priv->db,
 						   shell->priv->ui_component,
@@ -1245,17 +1366,17 @@ rb_shell_construct (RBShell *shell)
 	rb_sourcelist_set_dnd_targets (RB_SOURCELIST (shell->priv->sourcelist), target_table,
 				       G_N_ELEMENTS (target_table));
 
-	g_signal_connect (G_OBJECT (shell->priv->sourcelist), "selected",
-			  G_CALLBACK (source_selected_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->sourcelist), "selected",
+				 G_CALLBACK (source_selected_cb), shell, 0);
 
 	vbox = gtk_vbox_new (FALSE, 4);
 	shell->priv->notebook = gtk_notebook_new ();
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (shell->priv->notebook), FALSE);
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (shell->priv->notebook), FALSE);
-	g_signal_connect (G_OBJECT (shell->priv->notebook),
-			  "size_allocate",
-			  G_CALLBACK (paned_size_allocate_cb),
-			  shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->notebook),
+				 "size_allocate",
+				 G_CALLBACK (paned_size_allocate_cb),
+				 shell, 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (shell->priv->source_header), FALSE, TRUE, 0);
 	gtk_box_pack_start_defaults (GTK_BOX (vbox), shell->priv->notebook);
@@ -1282,6 +1403,9 @@ rb_shell_construct (RBShell *shell)
 
 	rb_debug ("shell: adding gconf notification");
 	/* sync state */
+	eel_gconf_notification_add (CONF_STATE_SELECTED_SOURCE,
+				    (GConfClientNotifyFunc) selected_source_changed_cb,
+				    shell);
 	eel_gconf_notification_add (CONF_UI_SOURCELIST_HIDDEN,
 				    (GConfClientNotifyFunc) sourcelist_visibility_changed_cb,
 				    shell);
@@ -1299,41 +1423,56 @@ rb_shell_construct (RBShell *shell)
 	shell->priv->show_db_errors = FALSE;
 	gtk_widget_hide (shell->priv->load_error_dialog);
 
-	g_signal_connect (G_OBJECT (shell->priv->db), "error",
-			  G_CALLBACK (rb_shell_db_error_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->db), "error",
+				 G_CALLBACK (rb_shell_db_error_cb), shell, 0);
 
-	g_signal_connect (G_OBJECT (shell->priv->load_error_dialog), "response",
-			  G_CALLBACK (rb_shell_load_failure_dialog_response_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->load_error_dialog), "response",
+				 G_CALLBACK (rb_shell_load_failure_dialog_response_cb), shell, 0);
 
 	/* initialize sources */
-	rb_debug ("shell: creating library source");
-	shell->priv->library_source = rb_library_source_new (shell->priv->db, shell->priv->ui_component);
-	rb_shell_append_source (shell, shell->priv->library_source);
+	while (known_sources[i] != NULL) {
+		RBSource *source;
 
-	rb_debug ("shell: creating iradio source");
-	shell->priv->iradio_source = RB_IRADIO_SOURCE (rb_iradio_source_new (shell->priv->db));
-	
-	rb_shell_append_source (shell, RB_SOURCE (shell->priv->iradio_source));
+		source = known_sources[i] (shell, 
+					   shell->priv->db, 
+					   shell->priv->ui_component);
 
+		rb_shell_append_source (shell, RB_SOURCE (source));
+		i++;
+	}
+
+	library_source = rb_shell_get_source_by_entry_type (shell, RHYTHMDB_ENTRY_TYPE_SONG);
+
+	rb_library_source_class_add_verbs (shell, shell->priv->ui_component);
+
+
+	g_assert (library_source != NULL);
+	iradio_source  = rb_shell_get_source_by_entry_type (shell, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION);
+	g_assert (iradio_source != NULL);
+
+	/* Initialize playlist manager */
+	rb_debug ("shell: creating playlist manager");
 	shell->priv->playlist_manager = rb_playlist_manager_new (shell->priv->ui_component,
 								 GTK_WINDOW (shell->priv->window),
 								 shell->priv->db,
 								 RB_SOURCELIST (shell->priv->sourcelist),
-								 RB_LIBRARY_SOURCE (shell->priv->library_source),
-								 RB_IRADIO_SOURCE (shell->priv->iradio_source));
+								 RB_LIBRARY_SOURCE (library_source),
+								 RB_IRADIO_SOURCE (iradio_source));
 
-	g_signal_connect (G_OBJECT (shell->priv->playlist_manager), "playlist_added",
-			  G_CALLBACK (rb_shell_playlist_added_cb), shell);
-	g_signal_connect (G_OBJECT (shell->priv->playlist_manager), "load_start",
-			  G_CALLBACK (rb_shell_playlist_load_start_cb), shell);
-	g_signal_connect (G_OBJECT (shell->priv->playlist_manager), "load_finish",
-			  G_CALLBACK (rb_shell_playlist_load_finish_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->playlist_manager), "playlist_added",
+				 G_CALLBACK (rb_shell_playlist_added_cb), shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->playlist_manager), "load_start",
+				 G_CALLBACK (rb_shell_playlist_load_start_cb), shell, 0);
+	g_signal_connect_object (G_OBJECT (shell->priv->playlist_manager), "load_finish",
+				 G_CALLBACK (rb_shell_playlist_load_finish_cb), shell, 0);
 
+	g_timeout_add (10000, (GSourceFunc) idle_save_playlist_manager, shell->priv->playlist_manager);
+		
 	rb_shell_sync_window_state (shell);
 
 	bonobo_ui_component_thaw (shell->priv->ui_component, NULL);
 
-	rb_shell_select_source (shell, shell->priv->library_source); /* select this one by default */
+	rb_shell_select_source_internal (shell, library_source);
 
 #ifdef HAVE_AUDIOCD
         if (rb_audiocd_is_any_device_available () == TRUE) {
@@ -1349,8 +1488,8 @@ rb_shell_construct (RBShell *shell)
 		else
 			rb_debug("CD is not available");
 
-		g_signal_connect (G_OBJECT (shell->priv->cd), "cd_changed",
-				  G_CALLBACK (audiocd_changed_cb), shell);
+		g_signal_connect_object (G_OBJECT (shell->priv->cd), "cd_changed",
+					 G_CALLBACK (audiocd_changed_cb), shell, 0);
         }
 	else
 		rb_debug ("No AudioCD device is available!");
@@ -1385,6 +1524,16 @@ rb_shell_construct (RBShell *shell)
 	}
 
 	/* GO GO GO! */
+	if (rhythmdb_exists) {
+		rb_debug ("loading database");
+		rhythmdb_load (shell->priv->db);
+/* Disabled for now */
+#if 0
+		rb_debug ("adding db save-when-needed thread");
+		g_timeout_add (10000, (GSourceFunc) idle_save_rhythmdb, shell->priv->db);
+#endif
+	}
+	
 	rb_debug ("shell: syncing window state");
 	rb_shell_sync_paned (shell);
 	gtk_widget_show_all (GTK_WIDGET (shell->priv->tray_icon));
@@ -1559,15 +1708,39 @@ rb_shell_load_failure_dialog_response_cb (GtkDialog *dialog,
 	shell->priv->show_db_errors = FALSE;
 }
 
+static RBSource *
+rb_shell_get_source_by_entry_type (RBShell *shell, RhythmDBEntryType type)
+{
+	return g_hash_table_lookup (shell->priv->sources_hash, (gpointer)type);
+}
+
+void
+rb_shell_register_entry_type_for_source (RBShell *shell,
+					 RBSource *source,
+					 RhythmDBEntryType type)
+{
+	if (shell->priv->sources_hash == NULL) {
+		shell->priv->sources_hash = g_hash_table_new (g_direct_hash, 
+							      g_direct_equal);
+	}
+	g_assert (g_hash_table_lookup (shell->priv->sources_hash, GINT_TO_POINTER (type)) == NULL);
+	g_hash_table_insert (shell->priv->sources_hash, 
+			     GINT_TO_POINTER (type), source);
+}
+
+
+
 static void
 rb_shell_append_source (RBShell *shell,
 			RBSource *source)
 {
+	char *search_text;
+	
 	shell->priv->sources
 		= g_list_append (shell->priv->sources, source);
 
-	g_signal_connect (G_OBJECT (source), "deleted",
-			  G_CALLBACK (rb_shell_source_deleted_cb), shell);
+	g_signal_connect_object (G_OBJECT (source), "deleted",
+				 G_CALLBACK (rb_shell_source_deleted_cb), shell, 0);
 
 	gtk_notebook_append_page (GTK_NOTEBOOK (shell->priv->notebook),
 				  GTK_WIDGET (source),
@@ -1576,6 +1749,12 @@ rb_shell_append_source (RBShell *shell,
 
 	rb_sourcelist_append (RB_SOURCELIST (shell->priv->sourcelist),
 			      source);
+
+	if (rb_source_can_search (source)) {
+		search_text = eel_gconf_get_string (rb_source_get_search_key (source));
+		rb_source_search (source, search_text);
+		g_free (search_text);
+	}
 }
 
 static void
@@ -1604,7 +1783,14 @@ rb_shell_source_deleted_cb (RBSource *source,
 		rb_shell_player_set_playing_source (shell->priv->player_shell, NULL);
 	}
 	if (source == shell->priv->selected_source) {
-		rb_shell_select_source (shell, shell->priv->library_source);
+		RBSource *library_source;
+		library_source = rb_shell_get_source_by_entry_type (shell, 
+								    RHYTHMDB_ENTRY_TYPE_SONG);
+		/* Set the gconf key */
+		rb_shell_select_source (shell, library_source);
+		/* Deal with it immediately, since we can't reference
+		 * the old source anymore. */
+		rb_shell_select_source_internal (shell, library_source);
 	}
 
 	shell->priv->sources = g_list_remove (shell->priv->sources, source);
@@ -1619,6 +1805,16 @@ rb_shell_source_deleted_cb (RBSource *source,
 static void
 rb_shell_select_source (RBShell *shell,
 			RBSource *source)
+{
+	char *internalname;
+
+	g_object_get (G_OBJECT (source), "internal-name", &internalname, NULL);
+	eel_gconf_set_string (CONF_STATE_SELECTED_SOURCE, internalname);
+}
+
+static void
+rb_shell_select_source_internal (RBShell *shell,
+				 RBSource *source)
 {
 	RBEntryView *view;
 
@@ -1636,15 +1832,21 @@ rb_shell_select_source (RBShell *shell,
 	shell->priv->selected_source = source;
 	
 	view = rb_source_get_entry_view (shell->priv->selected_source);
-	g_signal_connect (view, "notify::playing-entry", 
-			  G_CALLBACK(rb_shell_entry_changed_cb), shell);
+	g_signal_connect_object (view, "notify::playing-entry", 
+				 G_CALLBACK(rb_shell_entry_changed_cb), shell, 0);
 
 	/* show source */
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (shell->priv->notebook),
 				       gtk_notebook_page_num (GTK_NOTEBOOK (shell->priv->notebook), GTK_WIDGET (source)));
 
+	g_signal_handlers_block_by_func (G_OBJECT (shell->priv->sourcelist),
+					 G_CALLBACK (source_selected_cb),
+					 shell);
 	rb_sourcelist_select (RB_SOURCELIST (shell->priv->sourcelist),
 			      source);
+	g_signal_handlers_unblock_by_func (G_OBJECT (shell->priv->sourcelist),
+					   G_CALLBACK (source_selected_cb),
+					   shell);
 	
 	/* update services */
 	rb_shell_clipboard_set_source (shell->priv->clipboard_shell,
@@ -1901,6 +2103,11 @@ ask_file_response_cb (GtkDialog *dialog,
 #ifndef HAVE_GTK_2_3
 	char **files, **filecur;
 
+	if (response_id != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		return;
+	}
+
 	files = gtk_file_selection_get_selections (GTK_FILE_SELECTION (dialog));
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -1932,11 +2139,14 @@ ask_file_response_cb (GtkDialog *dialog,
 	shell->priv->show_db_errors = TRUE;
     
 	while (*filecur != NULL) {
-		if (g_utf8_validate (*filecur, -1, NULL)) {
+		char *utf8ized = g_filename_to_utf8 (*filecur, -1,
+						     NULL, NULL, NULL);
+		if (utf8ized) {
 			char *uri = gnome_vfs_get_uri_from_local_path (*filecur);
 			rhythmdb_add_uri_async (shell->priv->db, uri);
 			g_free (uri);
 		}
+		g_free (utf8ized);
 		filecur++;
 	}
 
@@ -1951,23 +2161,27 @@ ask_file_response_cb (GtkDialog *dialog,
 		return;
 	}
 
-	uri_list = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (dialog));
+	current_dir = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dialog));
+	eel_gconf_set_string (CONF_STATE_ADD_DIR, current_dir);
 
+
+	uri_list = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (dialog));
 	if (uri_list == NULL) {
-		current_dir = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dialog));
-		uri_list = g_slist_append (uri_list, current_dir);
+		uri_list = g_slist_append (uri_list, g_strdup (current_dir));
 	}
-	
+
 	shell->priv->show_db_errors = TRUE;
 
 	for (uris = uri_list; uris; uris = uris->next) {
 		if (g_utf8_validate ((char *)uris->data, -1, NULL)) 
 			rhythmdb_add_uri_async (shell->priv->db, (char *)uris->data);
 	}
+	g_slist_foreach (uri_list, (GFunc)g_free, NULL);
 	g_slist_free (uri_list);
 	g_free (current_dir);
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 #endif
+	g_timeout_add (10000, (GSourceFunc) idle_save_rhythmdb, shell->priv->db);
 }
 
 static void
@@ -1979,15 +2193,15 @@ rb_shell_cmd_add_to_library (BonoboUIComponent *component,
 	GtkWidget *dialog;
     
 	stored = eel_gconf_get_string (CONF_STATE_ADD_DIR);
-	dialog = rb_ask_file_multiple (_("Choose Files or Directory"),
+	dialog = rb_ask_dir_multiple (_("Choose Files or Directory"),
 				      stored,
 			              GTK_WINDOW (shell->priv->window));
 	g_free (stored);
 
-	g_signal_connect (G_OBJECT (dialog),
-			  "response",
-			  G_CALLBACK (ask_file_response_cb),
-				shell);
+	g_signal_connect_object (G_OBJECT (dialog),
+				 "response",
+				 G_CALLBACK (ask_file_response_cb),
+				 shell, 0);
 }
 
 static void
@@ -1995,9 +2209,13 @@ rb_shell_cmd_add_location (BonoboUIComponent *component,
 			   RBShell *shell,
 			   const char *verbname)
 {
+	RBSource *library_source;
+
 	shell->priv->show_db_errors = TRUE;
 
-	rb_library_source_add_location (RB_LIBRARY_SOURCE (shell->priv->library_source),
+	library_source = rb_shell_get_source_by_entry_type (shell, 
+							    RHYTHMDB_ENTRY_TYPE_SONG);
+	rb_library_source_add_location (RB_LIBRARY_SOURCE (library_source),
 					GTK_WINDOW (shell->priv->window));
 }
 
@@ -2038,6 +2256,7 @@ rb_shell_quit (RBShell *shell)
 {
 	rb_debug ("Quitting");
 
+	rb_shell_shutdown (shell);
 	rb_shell_sync_state (shell);
 	bonobo_object_unref (BONOBO_OBJECT (shell));
 }
@@ -2047,7 +2266,11 @@ rb_shell_load_complete_cb (RhythmDB *db, RBShell *shell)
 {
 	rb_debug ("load complete");
 	GDK_THREADS_ENTER ();
+
 	rb_playlist_manager_load_playlists (shell->priv->playlist_manager);
+	rb_shell_sync_selected_source (shell);
+	shell->priv->load_complete = TRUE;
+
 	GDK_THREADS_LEAVE ();
 }
 
@@ -2177,6 +2400,37 @@ paned_changed_cb (GConfClient *client,
 	rb_shell_sync_paned (shell);
 }
 
+static void
+rb_shell_sync_selected_source (RBShell *shell)
+{
+	char *internalname;
+	GList *tmp;
+
+	internalname = eel_gconf_get_string (CONF_STATE_SELECTED_SOURCE);
+	g_return_if_fail (internalname);
+
+	for (tmp = shell->priv->sources; tmp ; tmp = tmp->next) {
+		const char *tmpname;
+		g_object_get (G_OBJECT (tmp->data), "internal-name", &tmpname, NULL);
+		if (!strcmp (internalname, tmpname)) {
+			g_assert (tmp->data != NULL);
+			rb_shell_select_source_internal (shell, tmp->data);
+			return;
+		}
+	}
+	g_warning ("unknown source %s!", internalname);
+	rb_shell_select_source_internal (shell, rb_shell_get_source_by_entry_type (shell, RHYTHMDB_ENTRY_TYPE_SONG));
+}
+
+
+static void
+selected_source_changed_cb (GConfClient *client,
+			    guint cnxn_id,
+			    GConfEntry *entry,
+			    RBShell *shell)
+{
+	rb_shell_sync_selected_source (shell);
+}
 
 static void
 sourcelist_drag_received_cb (RBSourceList *sourcelist,
@@ -2224,10 +2478,18 @@ rb_shell_jump_to_entry_with_source (RBShell *shell, RBSource *source,
 		location = rhythmdb_entry_get_string (shell->priv->db, entry,
 						      RHYTHMDB_PROP_LOCATION);
 		rhythmdb_read_unlock (shell->priv->db);
-		if (rb_uri_is_iradio (location))
-			source = RB_SOURCE (shell->priv->iradio_source);
-		else
-			source = RB_SOURCE (shell->priv->library_source);
+		if (rb_uri_is_iradio (location)) {
+
+			RBSource *iradio_source;
+			iradio_source = rb_shell_get_source_by_entry_type (shell, 
+									    RHYTHMDB_ENTRY_TYPE_IRADIO_STATION);
+			source = RB_SOURCE (iradio_source);
+		} else {
+			RBSource *library_source;
+			library_source = rb_shell_get_source_by_entry_type (shell, 
+									    RHYTHMDB_ENTRY_TYPE_SONG);			
+			source = RB_SOURCE (library_source);
+		}
 	}
 
 	songs = rb_source_get_entry_view (source);
@@ -2300,8 +2562,8 @@ tray_deleted_cb (GtkWidget *win, GdkEventAny *event, RBShell *shell)
 						   shell->priv->ui_component,
 						   shell->priv->db,
 						   GTK_WINDOW (shell->priv->window));
-	g_signal_connect (G_OBJECT (shell->priv->tray_icon), "delete_event",
-			  G_CALLBACK (tray_deleted_cb), shell);
+	g_signal_connect_object (G_OBJECT (shell->priv->tray_icon), "delete_event",
+				 G_CALLBACK (tray_deleted_cb), shell, 0);
 }
 
 static void 
@@ -2322,7 +2584,7 @@ save_yourself_cb (GnomeClient *client,
                   RBShell *shell)
 {
         rb_debug ("session save yourself");
-        rb_shell_sync_state (shell);
+	rb_shell_sync_state (shell);
         return TRUE;
 }
 
@@ -2333,13 +2595,13 @@ rb_shell_session_init (RBShell *shell)
 
         client = gnome_master_client ();
 
-        g_signal_connect (G_OBJECT (client), 
-                          "save_yourself",
-                          G_CALLBACK (save_yourself_cb),
-                          shell);
+        g_signal_connect_object (G_OBJECT (client), 
+				 "save_yourself",
+				 G_CALLBACK (save_yourself_cb),
+				 shell, 0);
 
-        g_signal_connect (G_OBJECT (client),
-                          "die",
-                          G_CALLBACK (session_die_cb),
-                          shell);
+        g_signal_connect_object (G_OBJECT (client),
+				 "die",
+				 G_CALLBACK (session_die_cb),
+				 shell, 0);
 }
