@@ -20,9 +20,10 @@
  */
 
 #include <gtk/gtkhbox.h>
-#include <monkey-media.h>
 
+#include "rb-shell-player.h"
 #include "rb-source.h"
+#include "rhythmdb.h"
 
 #ifndef __RB_STATUSBAR_H
 #define __RB_STATUSBAR_H
@@ -52,7 +53,9 @@ typedef struct
 
 GType			rb_statusbar_get_type	(void);
 
-RBStatusbar *		rb_statusbar_new	(BonoboUIComponent *component);
+RBStatusbar *		rb_statusbar_new	(RhythmDB *db,
+						 BonoboUIComponent *component,
+						 RBShellPlayer *player);
 
 void			rb_statusbar_set_source	(RBStatusbar *bar,
 						 RBSource *player);
