@@ -42,6 +42,7 @@
 #include "rb-bonobo-helpers.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
+#include "rb-preferences.h"
 
 static void rb_tray_icon_class_init (RBTrayIconClass *klass);
 static void rb_tray_icon_init (RBTrayIcon *shell_player);
@@ -57,6 +58,8 @@ static void rb_tray_icon_get_property (GObject *object,
 static void rb_tray_set_visibility (RBTrayIcon *tray, int state);
 static void rb_tray_icon_button_press_event_cb (GtkWidget *ebox, GdkEventButton *event,
 						RBTrayIcon *icon);
+static void rb_tray_icon_scroll_event_cb (GtkWidget *ebox, GdkEvent *event,
+						RBTrayIcon *icon);
 static void sync_menu (RBTrayIcon *tray);
 static void rb_tray_icon_show_window_changed_cb (BonoboUIComponent *component,
 					const char *path,
@@ -88,6 +91,12 @@ struct RBTrayIconPrivate
 	GtkWidget *ebox;
 
 	RhythmDB *db;
+
+	int window_x;
+	int window_y;
+	int window_w;
+	int window_h;
+	gboolean visible;
 };
 
 enum
@@ -218,13 +227,17 @@ rb_tray_icon_init (RBTrayIcon *icon)
 			      _("Not playing"), NULL);
 
 	icon->priv->ebox = gtk_event_box_new ();
-	g_signal_connect (G_OBJECT (icon->priv->ebox),
-			  "button_press_event",
-			  G_CALLBACK (rb_tray_icon_button_press_event_cb),
-			  icon);
+	g_signal_connect_object (G_OBJECT (icon->priv->ebox),
+				 "button_press_event",
+				 G_CALLBACK (rb_tray_icon_button_press_event_cb),
+				 icon, 0);
+	g_signal_connect_object (G_OBJECT (icon->priv->ebox),
+				 "scroll_event",
+				 G_CALLBACK (rb_tray_icon_scroll_event_cb),
+				 icon, 0);
 	gtk_drag_dest_set (icon->priv->ebox, GTK_DEST_DEFAULT_ALL, target_uri, 1, GDK_ACTION_COPY);
-	g_signal_connect (G_OBJECT (icon->priv->ebox), "drag_data_received",
-			  G_CALLBACK (rb_tray_icon_drop_cb), icon);
+	g_signal_connect_object (G_OBJECT (icon->priv->ebox), "drag_data_received",
+				 G_CALLBACK (rb_tray_icon_drop_cb), icon, 0);
 
 	image = gtk_image_new_from_stock (RB_STOCK_TRAY_ICON,
 					  GTK_ICON_SIZE_SMALL_TOOLBAR);
@@ -233,6 +246,12 @@ rb_tray_icon_init (RBTrayIcon *icon)
 	icon->priv->control = bonobo_control_new (icon->priv->ebox);
 	icon->priv->tray_component = bonobo_control_get_popup_ui_component (icon->priv->control);
 
+	icon->priv->window_x = -1;
+	icon->priv->window_y = -1;
+	icon->priv->window_w = -1;
+	icon->priv->window_h = -1;
+	icon->priv->visible = TRUE;
+
 	rb_bonobo_add_listener_list_with_data (icon->priv->tray_component,
 						rb_tray_icon_listeners,
 						icon);
@@ -370,6 +389,33 @@ rb_tray_icon_button_press_event_cb (GtkWidget *ebox, GdkEventButton *event,
 	}
 }
 
+static void
+rb_tray_icon_scroll_event_cb (GtkWidget *ebox, GdkEvent *event,
+				    RBTrayIcon *icon)
+{
+	float volume;
+	rb_debug ("tray button scroll");
+	volume = eel_gconf_get_float (CONF_STATE_VOLUME);
+	switch(event->scroll.direction) {
+	case GDK_SCROLL_UP:
+		volume += 0.1;
+		if (volume > 1.0)
+			volume = 1.0;
+		break;
+	case GDK_SCROLL_DOWN:
+		volume -= 0.1;
+		if (volume < 0)
+			volume = 0;
+		break;
+	case GDK_SCROLL_LEFT:
+	case GDK_SCROLL_RIGHT:
+		break;
+	}
+	
+	rb_debug ("got scroll, setting volume to %f", volume);
+	eel_gconf_set_float (CONF_STATE_VOLUME, volume);
+}
+
 static void
 rb_tray_icon_drop_cb (GtkWidget *widget,
 		      GdkDragContext *context,
@@ -467,35 +513,57 @@ rb_tray_icon_set_tooltip (RBTrayIcon *icon, const char *tooltip)
 }
 
 static void
-rb_tray_set_visibility (RBTrayIcon *tray, int state)
+rb_tray_restore_main_window (RBTrayIcon *icon)
 {
-	static int window_x = -1;
-	static int window_y = -1;
-	static gboolean visible = 1;
+	if ((icon->priv->window_x >= 0 && icon->priv->window_y >= 0) || (icon->priv->window_h >= 0 && icon->priv->window_w >=0 ))
+	{
+		gtk_widget_realize (GTK_WIDGET (icon->priv->main_window));
+		gdk_flush ();
 
+		if (icon->priv->window_x >= 0 && icon->priv->window_y >= 0)
+		{
+			gtk_window_move (icon->priv->main_window,
+					icon->priv->window_x,
+					icon->priv->window_y);                                }
+		if (icon->priv->window_w >= 0 && icon->priv->window_y >=0)
+		{
+			gtk_window_resize (icon->priv->main_window,
+					icon->priv->window_w,
+					icon->priv->window_h);
+		}
+	}
+}
+
+static void
+rb_tray_set_visibility (RBTrayIcon *icon, int state)
+{
 	switch (state)
 	{
 	case VISIBILITY_HIDDEN:
        	case VISIBILITY_VISIBLE:
-		visible = !state;
-
+		if (icon->priv->visible != state)
+			rb_tray_set_visibility (icon, VISIBILITY_TOGGLE);
+		break;
         case VISIBILITY_TOGGLE:
-		visible = !visible;
-	
-		if (visible == TRUE)
+		icon->priv->visible = !icon->priv->visible;
+
+		if (icon->priv->visible == TRUE)
 		{
-			if (window_x >= 0 && window_y >= 0)
-				gtk_window_move (tray->priv->main_window,
-						window_x, window_y );
-			gtk_widget_show (GTK_WIDGET (tray->priv->main_window));
+			rb_tray_restore_main_window (icon);
+			gtk_widget_show (GTK_WIDGET (icon->priv->main_window));
 		} else {
-			gtk_window_get_position ( tray->priv->main_window,
-						&window_x, &window_y);
-			gtk_widget_hide (GTK_WIDGET (tray->priv->main_window));
+			gtk_window_get_position (icon->priv->main_window,
+						 &icon->priv->window_x,
+						 &icon->priv->window_y);
+			gtk_window_get_size (icon->priv->main_window,
+					     &icon->priv->window_w,
+					     &icon->priv->window_h);
+			gtk_widget_hide (GTK_WIDGET (icon->priv->main_window));
 		}
 
 	case VISIBILITY_SYNC:
-		rb_bonobo_set_active (tray->priv->main_component,
-					CMD_PATH_SHOW_WINDOW, visible);
+		rb_bonobo_set_active (icon->priv->main_component,
+				      CMD_PATH_SHOW_WINDOW,
+				      icon->priv->visible);
 	}
 }
