@@ -31,7 +31,6 @@
 
 #include "rb-iradio-source.h"
 
-#include "rhythmdb-legacy.h"
 #include "rhythmdb-query-model.h"
 #include "rb-bonobo-helpers.h"
 #include "rb-glade-helpers.h"
@@ -93,6 +92,7 @@ static void rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view
 /* source methods */
 static const char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
+static const char *impl_get_search_key (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
 static RBEntryView *impl_get_entry_view (RBSource *source);
 static void impl_search (RBSource *source, const char *text);
@@ -116,6 +116,7 @@ void rb_iradio_source_show_columns_changed_cb (GtkToggleButton *button,
 #define CONF_STATE_PANED_POSITION CONF_PREFIX "/state/iradio/paned_position"
 #define CONF_STATE_IRADIO_SORTING CONF_PREFIX "/state/iradio/sorting"
 #define CONF_STATE_SHOW_BROWSER   CONF_PREFIX "/state/iradio/show_browser"
+#define CONF_STATE_SEARCH_TEXT   CONF_PREFIX "/state/iradio/search_text"
 
 struct RBIRadioSourcePrivate
 {
@@ -137,6 +138,8 @@ struct RBIRadioSourcePrivate
 	gboolean shuffle;
 	gboolean repeat;
 
+	gboolean initialized;
+
 	char *status;
 
 	GtkWidget *paned;
@@ -159,12 +162,15 @@ struct RBIRadioSourcePrivate
 	gboolean async_entry_destroyed;
 	guint async_signum;
 	RhythmDB *async_update_entry;
+
+	RhythmDBEntryType entry_type;
 };
 
 enum
 {
 	PROP_0,
 	PROP_DB,
+	PROP_ENTRY_TYPE
 };
 
 static GObjectClass *parent_class = NULL;
@@ -214,6 +220,7 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 
 	source_class->impl_get_status  = impl_get_status;
 	source_class->impl_get_browser_key  = impl_get_browser_key;
+	source_class->impl_get_search_key  = impl_get_search_key;
 	source_class->impl_get_pixbuf  = impl_get_pixbuf;
 	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_search = impl_search;
@@ -234,6 +241,15 @@ rb_iradio_source_class_init (RBIRadioSourceClass *klass)
 							      "RhythmDB database",
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+					 PROP_ENTRY_TYPE,
+					 g_param_spec_uint ("entry-type",
+							    "Entry type",
+							    "Type of the entries which should be displayed by this source",
+							    0,
+							    G_MAXINT,
+							    RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
+							    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	
 }
 
@@ -289,7 +305,7 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	RBIRadioSource *source;
 	RBIRadioSourceClass *klass;
 	GObjectClass *parent_class;  
-	klass = RB_IRADIO_SOURCE_CLASS (g_type_class_peek (type));
+	klass = RB_IRADIO_SOURCE_CLASS (g_type_class_peek (RB_TYPE_IRADIO_SOURCE));
 
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
 	source = RB_IRADIO_SOURCE (parent_class->constructor (type, n_construct_properties,
@@ -306,26 +322,26 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_LAST_PLAYED);
-	g_signal_connect (G_OBJECT (source->priv->stations),
-			  "sort-order-changed",
-			  G_CALLBACK (rb_iradio_source_songs_view_sort_order_changed_cb),
-			  source);
+	g_signal_connect_object (G_OBJECT (source->priv->stations),
+				 "sort-order-changed",
+				 G_CALLBACK (rb_iradio_source_songs_view_sort_order_changed_cb),
+				 source, 0);
 
-	g_signal_connect (G_OBJECT (source->priv->stations),
-			  "size_allocate",
-			  G_CALLBACK (paned_size_allocate_cb),
-			  source);
-	g_signal_connect (G_OBJECT (source->priv->stations), "show_popup",
-			  G_CALLBACK (rb_iradio_source_songs_show_popup_cb), source);
+	g_signal_connect_object (G_OBJECT (source->priv->stations),
+				 "size_allocate",
+				 G_CALLBACK (paned_size_allocate_cb),
+				 source, 0);
+	g_signal_connect_object (G_OBJECT (source->priv->stations), "show_popup",
+				 G_CALLBACK (rb_iradio_source_songs_show_popup_cb), source, 0);
 
 	/* set up genre entry view */
 	source->priv->genres = rb_property_view_new (source->priv->db,
 						     RHYTHMDB_PROP_GENRE,
 						     _("Genre"));
-	g_signal_connect (G_OBJECT (source->priv->genres),
-			  "property-selected",
-			  G_CALLBACK (genre_selected_cb),
-			  source);
+	g_signal_connect_object (G_OBJECT (source->priv->genres),
+				 "property-selected",
+				 G_CALLBACK (genre_selected_cb),
+				 source, 0);
 
 	g_object_set (G_OBJECT (source->priv->genres), "vscrollbar_policy",
 		      GTK_POLICY_AUTOMATIC, NULL);
@@ -344,9 +360,10 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	eel_gconf_notification_add (CONF_FIRST_TIME,
 				    (GConfClientNotifyFunc) rb_iradio_source_first_time_changed,
 				    source);
-	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_ALL);
-			
 	gtk_widget_show_all (GTK_WIDGET (source));
+
+	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_ALL);
+
 	return G_OBJECT (source);
 }
 
@@ -363,6 +380,9 @@ rb_iradio_source_set_property (GObject *object,
 	case PROP_DB:
 		source->priv->db = g_value_get_object (value);
 		break;
+	case PROP_ENTRY_TYPE:
+		source->priv->entry_type = g_value_get_uint (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -382,6 +402,9 @@ rb_iradio_source_get_property (GObject *object,
 	case PROP_DB:
 		g_value_set_object (value, source->priv->db);
 		break;
+	case PROP_ENTRY_TYPE:
+		g_value_set_uint (value, source->priv->entry_type);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -389,15 +412,21 @@ rb_iradio_source_get_property (GObject *object,
 }
 
 RBSource *
-rb_iradio_source_new (RhythmDB *db)
+rb_iradio_source_new (RBShell *shell, RhythmDB *db, 
+		      BonoboUIComponent *component)
 {
 	RBSource *source;
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_IRADIO_SOURCE,
 					  "name", _("Radio"),
+					  "internal-name", "<radio>",
 					  "db", db,
+					  "component", component,
 					  NULL));
 
+	rb_shell_register_entry_type_for_source (shell, source, 
+						 RHYTHMDB_ENTRY_TYPE_IRADIO_STATION);
+
 	return source;
 }
 
@@ -427,7 +456,12 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 		rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
 	}
 	g_value_unset (&val);
-		
+	
+	g_value_init (&val, G_TYPE_DOUBLE);
+	g_value_set_double (&val, 2.5);
+	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_RATING, &val);
+	g_value_unset (&val);
+	
 	rhythmdb_write_unlock (source->priv->db);
 }
 
@@ -443,12 +477,17 @@ static void
 impl_search (RBSource *asource, const char *search_text)
 {
 	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
-	if (search_text == NULL && source->priv->search_text == NULL)
-		return;
-	if (search_text != NULL &&
-	    source->priv->search_text != NULL
-	    && !strcmp (search_text, source->priv->search_text))
-		return;
+
+	if (source->priv->initialized) {
+		if (search_text == NULL && source->priv->search_text == NULL)
+			return;
+		if (search_text != NULL &&
+		    source->priv->search_text != NULL
+		    && !strcmp (search_text, source->priv->search_text))
+			return;
+	}
+
+	source->priv->initialized = TRUE;
 	if (search_text != NULL && search_text[0] == '\0')
 		search_text = NULL;
 
@@ -546,6 +585,12 @@ impl_get_browser_key (RBSource *asource)
 	return CONF_STATE_SHOW_BROWSER;
 }
 
+static const char *
+impl_get_search_key (RBSource *asource)
+{
+	return CONF_STATE_SEARCH_TEXT;
+}
+
 static void
 impl_delete (RBSource *asource)
 {
@@ -608,6 +653,7 @@ rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view,
 						   RBIRadioSource *source)
 {
 	rb_debug ("sort order changed");
+	rb_entry_view_set_resorting (view);
 	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_SEARCH);
 }
 
@@ -665,17 +711,19 @@ rb_iradio_source_show_browser (RBIRadioSource *source,
 static void
 rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType qtype)
 {
+	RhythmDBEntryType entry_type;
 	RhythmDBQueryModel *query_model;
 	RhythmDBPropertyModel *genre_model;
 	GtkTreeModel *model;
 	GPtrArray *query;
 
+	g_object_get (G_OBJECT (source), "entry-type", &entry_type, NULL);
 	rhythmdb_read_lock (source->priv->db);
 	source->priv->query_type = qtype;
 	query = rhythmdb_query_parse (source->priv->db,
 				      RHYTHMDB_QUERY_PROP_EQUALS,
 				      RHYTHMDB_PROP_TYPE,
-				      RHYTHMDB_ENTRY_TYPE_IRADIO_STATION,
+				      entry_type,
 				      RHYTHMDB_QUERY_END);
 
 	if (source->priv->search_text) {
